﻿// --------------------------------------------------------------------------------------------
// Stefan Cruysberghs
// http://www.scip.be
// Article: Customize WCF RIA Services code generation
// http://www.scip.be/index.php?Page=ArticlesNET36
// --------------------------------------------------------------------------------------------
using System;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Diagnostics;

namespace ScipBe.Demo.RiaCodeGen.Web.Generators
{
    public class XmlCommentsProvider
    {
        private XElement comments;

        public XmlCommentsProvider(Type classType, string path)
        {
            //var assemblyName = classType.Assembly.GetName().Name;
            //var assemblyXmlFileName = string.Format("..\\{0}\\bin\\{0}.XML", assemblyName);
            var assemblyXmlFileName = path.Replace("dll", "xml");
            comments = XElement.Load(new FileStream(assemblyXmlFileName, FileMode.Open));
        }

        private XElement FindCommentElementByName(string name)
        {
            return comments
                .Descendants("member")
                .Where(x => String.Compare(x.Attributes("name").FirstOrDefault().Value.ToString(), name, true) == 0)
                .FirstOrDefault();
        }

        private string PrintInnerXml(XElement element)
        {
            string[] lines = element.Nodes()
                .Aggregate("", (b, node) => b += node.ToString())
                .Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);

            var sb = new StringBuilder();
            foreach (string line in lines)
            {
                sb.AppendLine(line.Trim().Insert(0, "///"));
            }
            return sb.ToString().Trim();
        }

        public string GetClassComments(Type classType)
        {
            var element = FindCommentElementByName("T:" + classType.FullName);

            if (element != null)
            {
                return PrintInnerXml(element);
            }

            return null;
        }

        public string GetPropertyComments(PropertyDescriptor propertyDescriptor)
        {
            var element = FindCommentElementByName("P:" + propertyDescriptor.ComponentType.FullName + "." + propertyDescriptor.Name);

            if (element != null)
            {
                return PrintInnerXml(element);
            }

            return null;
        }
    }
}